#include <afxwin.h>
#include "extapi.h"

// the one and only CExtAPI instance
CExtAPI theAPI;

CExtAPI::CExtAPI(void) {

	m_hID = NULL;

	// app functions

	m_lpfnGetFuncHandle = NULL;
	m_lpfnReportError = NULL;

	// var function handles

	m_lpfn_var_Create = NULL;
	m_lpfn_var_CreateTemp = NULL;
	m_lpfn_var_Delete = NULL;
	m_lpfn_var_Rename = NULL;
	m_lpfn_var_GetVar = NULL;
	m_lpfn_var_GetName = NULL;
	m_lpfn_var_GetTypeID = NULL;
	m_lpfn_var_IsType = NULL;
	m_lpfn_var_GetValue = NULL;
	m_lpfn_var_SetValue = NULL;
	m_lpfn_var_GetValueEx = NULL;
	m_lpfn_var_SetValueEx = NULL;
	m_lpfn_var_CopyValue = NULL;

	// type function handles

	m_lpfn_type_GetTypeID = NULL;
	m_lpfn_type_GetTypeName = NULL;

	// string function handles

	m_lpfn_str_GetText = NULL;
	m_lpfn_str_SetText = NULL;

	// list function handles

	m_lpfn_list_GetList = NULL;
	m_lpfn_list_nItems = NULL;
	m_lpfn_list_CreateItem = NULL;
	m_lpfn_list_GetItemI = NULL;
	m_lpfn_list_GetItemA = NULL;
	m_lpfn_list_DelItemI = NULL;
	m_lpfn_list_DelItemA = NULL;
	m_lpfn_list_DelAll = NULL;

	// map functions

	m_lpfn_map_GetMap = NULL;
	m_lpfn_map_Init = NULL;
	m_lpfn_map_Free = NULL;
	m_lpfn_map_GetMapType = NULL;
	m_lpfn_map_GetPixelSize = NULL;
	m_lpfn_map_GetWrapFlag = NULL;
	m_lpfn_map_GetWidth = NULL;
	m_lpfn_map_GetHeight = NULL;
	m_lpfn_map_GetPixel = NULL;
	m_lpfn_map_SetPixel = NULL;
	m_lpfn_map_GetMinMaxAlt = NULL;
	m_lpfn_map_GetHorizScale = NULL;
	m_lpfn_map_GetMosaicFlag = NULL;
	m_lpfn_map_GetMosaicTileSize = NULL;
	m_lpfn_map_GetMosaicTilesX = NULL;
	m_lpfn_map_GetMosaicTilesY = NULL;

	// tile functions

	m_lpfn_tile_GetMinMaxAlt = NULL;
	m_lpfn_tile_GetPixel = NULL;
	m_lpfn_tile_SetPixel = NULL;

	// format functions

	m_lpfn_format_Create = NULL;
	m_lpfn_format_CreateGeneric = NULL;
	m_lpfn_format_SetFlags = NULL;
	m_lpfn_format_GetExt = NULL;
	m_lpfn_format_GetOptionList = NULL;
	m_lpfn_format_GetOptionValue = NULL;
	m_lpfn_format_SetOptionValue = NULL;

	// prog box function handles

	m_lpfn_progbox_ShowWnd = NULL;
	m_lpfn_progbox_HideWnd = NULL;
	m_lpfn_progbox_SetTitle = NULL;
	m_lpfn_progbox_SetProgress = NULL;

	// combo box function handles

	m_lpfn_combosel_SetOptions = NULL;
	m_lpfn_combosel_GetOptions = NULL;
	m_lpfn_combosel_SetCurSel = NULL;
	m_lpfn_combosel_GetCurSel = NULL;

	// buffer function handles

	m_lpfn_buffer_Init = NULL;
	m_lpfn_buffer_Free = NULL;
	m_lpfn_buffer_nItems = NULL;
	m_lpfn_buffer_MemSize = NULL;
	m_lpfn_buffer_ItemSize = NULL;
	m_lpfn_buffer_GetPtr = NULL;
	m_lpfn_buffer_GetValue = NULL;
	m_lpfn_buffer_SetValue = NULL;

	// project function handle defs

	m_lpfn_project_GetHeightfieldRange = NULL;

	// calc function handle defs

	m_lpfn_calc_AM_GetPixelColour = NULL;
	m_lpfn_calc_AM_GetIDfromColour = NULL;

	// file/dir function handles

	m_lpfn_file_GetExt = NULL;
	m_lpfn_file_FileExists = NULL;

	// extension functions

	m_lpfn_zeofunc_LoadFunc = NULL;
	m_lpfn_zeofunc_LoadFuncEx = NULL;
	m_lpfn_zeofunc_GetFunc = NULL;
	m_lpfn_zeofunc_GetReturnTypeID = NULL;
	m_lpfn_zeofunc_GetArgListPrototype = NULL;
	m_lpfn_zeofunc_Execute = NULL;
}

CExtAPI::~CExtAPI(void) {
}

bool CExtAPI::InitAPI(FARPROC pFunc, LPVOID hID) {

	if(!pFunc) {
		AfxMessageBox("CL3DTio_HFFApp::InitAPI error:\r\n - Invalid callback handle");
		return false;
	}

	if(!hID) {
		AfxMessageBox("CL3DTio_HFFApp::InitAPI error:\r\n - Invalid ID handle");
		return false;
	}

	//store thr session ID
	m_hID = hID;

	// store the handle to the function by which we load more function handles
	m_lpfnGetFuncHandle = (LPFNAPI_app_GetFuncHandle)pFunc;

	// debugging
//	CString TempStr;
//	TempStr.Format("CExtAPI::InitAPI report:\r\n - GetFuncHandle is 0x%x", m_lpfnGetFuncHandle);
//	AfxMessageBox(TempStr);

	// start loading functions from L3DT

	//
	// app functions
	//

	m_lpfnReportError = (LPFNAPI_app_ReportError)GetFuncHandle("app.ReportError");

	//
	// var functions
	//

	m_lpfn_var_Create = (LPFNAPI_var_Create)GetFuncHandle("var.Create");
	m_lpfn_var_CreateTemp = (LPFNAPI_var_CreateTemp)GetFuncHandle("var.CreateTemp");
	m_lpfn_var_Delete = (LPFNAPI_var_Delete)GetFuncHandle("var.Delete");
	m_lpfn_var_Rename = (LPFNAPI_var_Rename)GetFuncHandle("var.Rename");
	m_lpfn_var_GetVar = (LPFNAPI_var_GetVar)GetFuncHandle("var.GetVar");
	m_lpfn_var_GetName = (LPFNAPI_var_GetName)GetFuncHandle("var.GetName");
	m_lpfn_var_GetTypeID = (LPFNAPI_var_GetTypeID)GetFuncHandle("var.GetTypeID");
	m_lpfn_var_IsType = (LPFNAPI_var_IsType)GetFuncHandle("var.IsType");
	m_lpfn_var_GetValue = (LPFNAPI_var_GetValue)GetFuncHandle("var.GetValue");
	m_lpfn_var_SetValue = (LPFNAPI_var_SetValue)GetFuncHandle("var.SetValue");
	m_lpfn_var_GetValueEx = (LPFNAPI_var_GetValueEx)GetFuncHandle("var.GetValueEx");
	m_lpfn_var_SetValueEx = (LPFNAPI_var_SetValueEx)GetFuncHandle("var.SetValueEx");
	m_lpfn_var_CopyValue = (LPFNAPI_var_CopyValue)GetFuncHandle("var.CopyValue");

	//
	// type functions
	//

	m_lpfn_type_GetTypeID = (LPFNAPI_type_GetTypeID)GetFuncHandle("type.GetTypeID");
	m_lpfn_type_GetTypeName = (LPFNAPI_type_GetTypeName)GetFuncHandle("type.GetTypeName");

	//
	// string functions
	//

	m_lpfn_str_GetText = (LPFNAPI_str_GetText)GetFuncHandle("str.GetText");
	m_lpfn_str_SetText = (LPFNAPI_str_SetText)GetFuncHandle("str.SetText");

	//
	// list functions
	//

	m_lpfn_list_GetList = (LPFNAPI_list_GetList)GetFuncHandle("list.GetList");
	m_lpfn_list_nItems = (LPFNAPI_list_nItems)GetFuncHandle("list.nItems");
	m_lpfn_list_CreateItem = (LPFNAPI_list_CreateItem)GetFuncHandle("list.CreateItem");
	m_lpfn_list_GetItemI = (LPFNAPI_list_GetItemI)GetFuncHandle("list.GetItemI");
	m_lpfn_list_GetItemA = (LPFNAPI_list_GetItemA)GetFuncHandle("list.GetItemA");
	m_lpfn_list_DelItemI = (LPFNAPI_list_DelItemI)GetFuncHandle("list.DelItemI");
	m_lpfn_list_DelItemA = (LPFNAPI_list_DelItemA)GetFuncHandle("list.DelItemA");
	m_lpfn_list_DelAll = (LPFNAPI_list_DelAll)GetFuncHandle("list.DelAll");

	//
	// map functions
	//

	m_lpfn_map_GetMap = (LPFNAPI_map_GetMap)GetFuncHandle("map.GetMap");
	m_lpfn_map_Init = (LPFNAPI_map_Init)GetFuncHandle("map.Init");
	m_lpfn_map_Free = (LPFNAPI_map_Free)GetFuncHandle("map.Free");
	m_lpfn_map_GetMapType = (LPFNAPI_map_GetPixelSize)GetFuncHandle("map.GetMapType");
	m_lpfn_map_GetPixelSize = (LPFNAPI_map_GetPixelSize)GetFuncHandle("map.GetPixelSize");
	m_lpfn_map_GetWrapFlag = (LPFNAPI_map_GetWrapFlag)GetFuncHandle("map.GetWrapFlag");
	m_lpfn_map_GetWidth = (LPFNAPI_map_GetWidth)GetFuncHandle("map.GetWidth");
	m_lpfn_map_GetHeight = (LPFNAPI_map_GetHeight)GetFuncHandle("map.GetHeight");
	m_lpfn_map_GetPixel = (LPFNAPI_map_GetPixel)GetFuncHandle("map.GetPixel");
	m_lpfn_map_SetPixel = (LPFNAPI_map_SetPixel)GetFuncHandle("map.SetPixel");
	m_lpfn_map_GetMinMaxAlt = (LPFNAPI_map_GetMinMaxAlt)GetFuncHandle("map.GetMinMaxAlt");
	m_lpfn_map_GetHorizScale = (LPFNAPI_map_GetHorizScale)GetFuncHandle("map.GetHorizScale");
	m_lpfn_map_GetMosaicFlag = (LPFNAPI_map_GetMosaicFlag)GetFuncHandle("map.GetMosaicFlag");
	m_lpfn_map_GetMosaicTileSize = (LPFNAPI_map_GetMosaicTileSize)GetFuncHandle("map.GetMosaicTileSize");
	m_lpfn_map_GetMosaicTilesX = (LPFNAPI_map_GetMosaicTilesX)GetFuncHandle("map.GetMosaicTilesX");
	m_lpfn_map_GetMosaicTilesY = (LPFNAPI_map_GetMosaicTilesY)GetFuncHandle("map.GetMosaicTilesY");

	//
	// mosaic tile functions
	//

	m_lpfn_tile_GetMinMaxAlt = (LPFNAPI_tile_GetMinMaxAlt)GetFuncHandle("tile.GetMinMaxAlt");
	m_lpfn_tile_GetPixel = (LPFNAPI_tile_GetPixel)GetFuncHandle("tile.GetPixel");
	m_lpfn_tile_SetPixel = (LPFNAPI_tile_SetPixel)GetFuncHandle("tile.SetPixel");

	//
	// format functions
	//

	m_lpfn_format_Create = (LPFNAPI_format_Create)GetFuncHandle("format.Create");
	m_lpfn_format_CreateGeneric = (LPFNAPI_format_CreateGeneric)GetFuncHandle("format.CreateGeneric");
	m_lpfn_format_SetFlags = (LPFNAPI_format_SetFlags)GetFuncHandle("format.SetFlags");
	m_lpfn_format_GetExt = (LPFNAPI_format_GetExt)GetFuncHandle("format.GetExt");
	m_lpfn_format_GetOptionList = (LPFNAPI_format_GetOptionList)GetFuncHandle("format.GetOptionList");
	m_lpfn_format_GetOptionValue = (LPFNAPI_format_GetOptionValue)GetFuncHandle("format.GetOptionValue");
	m_lpfn_format_SetOptionValue = (LPFNAPI_format_SetOptionValue)GetFuncHandle("format.SetOptionValue");

	//
	// prog box function handles
	//

	m_lpfn_progbox_ShowWnd = (LPFNAPI_progbox_ShowWnd)GetFuncHandle("progbox.ShowWnd");
	m_lpfn_progbox_HideWnd = (LPFNAPI_progbox_HideWnd)GetFuncHandle("progbox.HideWnd");
	m_lpfn_progbox_SetTitle = (LPFNAPI_progbox_SetTitle)GetFuncHandle("progbox.SetTitle");
	m_lpfn_progbox_SetProgress = (LPFNAPI_progbox_SetProgress)GetFuncHandle("progbox.SetProgress");

	//
	// combo box function handles
	//

	m_lpfn_combosel_SetOptions = (LPFNAPI_combosel_SetOptions)GetFuncHandle("combosel.SetOptions");
	m_lpfn_combosel_GetOptions = (LPFNAPI_combosel_GetOptions)GetFuncHandle("combosel.GetOptions");
	m_lpfn_combosel_SetCurSel = (LPFNAPI_combosel_SetCurSel)GetFuncHandle("combosel.SetCurSel");
	m_lpfn_combosel_GetCurSel = (LPFNAPI_combosel_GetCurSel)GetFuncHandle("combosel.GetCurSel");

	//
	// buffer function handles
	//

	m_lpfn_buffer_Init = (LPFNAPI_buffer_Init)GetFuncHandle("buffer.Init");
	m_lpfn_buffer_Free = (LPFNAPI_buffer_Free)GetFuncHandle("buffer.Free");
	m_lpfn_buffer_nItems = (LPFNAPI_buffer_nItems)GetFuncHandle("buffer.nItems");
	m_lpfn_buffer_MemSize = (LPFNAPI_buffer_MemSize)GetFuncHandle("buffer.MemSize");
	m_lpfn_buffer_ItemSize = (LPFNAPI_buffer_ItemSize)GetFuncHandle("buffer.ItemSize");
	m_lpfn_buffer_GetPtr = (LPFNAPI_buffer_GetPtr)GetFuncHandle("buffer.GetPtr");
	m_lpfn_buffer_GetValue = (LPFNAPI_buffer_GetValue)GetFuncHandle("buffer.GetValue");
	m_lpfn_buffer_SetValue = (LPFNAPI_buffer_SetValue)GetFuncHandle("buffer.SetValue");

	//
	// project function handles
	//

	m_lpfn_project_GetHeightfieldRange = (LPFNAPI_project_GetHeightfieldRange)GetFuncHandle("project.GetHeightfieldRange");

	//
	// calc funtion handles
	//

	m_lpfn_calc_AM_GetPixelColour = (LPFNAPI_calc_AM_GetPixelColour)GetFuncHandle("calc.AM.GetPixelColour");
	m_lpfn_calc_AM_GetIDfromColour = (LPFNAPI_calc_AM_GetIDfromColour)GetFuncHandle("calc.AM.GetIDfromColour");

	//
	// file/dir function handles
	//

	m_lpfn_file_GetExt = (LPFNAPI_file_GetExt)GetFuncHandle("file.GetExt");
	m_lpfn_file_FileExists = (LPFNAPI_file_FileExists)GetFuncHandle("file.FileExists");

	//
	// extension function handles
	//

	m_lpfn_zeofunc_LoadFunc = (LPFNAPI_zeofunc_LoadFunc)GetFuncHandle("zeofunc.LoadFunc");
	m_lpfn_zeofunc_LoadFuncEx = (LPFNAPI_zeofunc_LoadFuncEx)GetFuncHandle("zeofunc.LoadFuncEx");
	m_lpfn_zeofunc_GetFunc = (LPFNAPI_zeofunc_GetFunc)GetFuncHandle("zeofunc.GetFunc");
	m_lpfn_zeofunc_GetReturnTypeID = (LPFNAPI_zeofunc_GetReturnTypeID)GetFuncHandle("zeofunc.GetReturnTypeID");
	m_lpfn_zeofunc_GetArgListPrototype = (LPFNAPI_zeofunc_GetArgListPrototype)GetFuncHandle("zeofunc.GetArgListPrototype");
	m_lpfn_zeofunc_Execute = (LPFNAPI_zeofunc_Execute)GetFuncHandle("zeofunc.Execute");

	return true;
}

//
// app functions
//

void CExtAPI::ReportError(LPCSTR lpErrorMessage) {
	
	if(!m_lpfnReportError) {
		AfxMessageBox(lpErrorMessage, MB_ICONERROR);
	}

	m_lpfnReportError(m_hID, lpErrorMessage);
}

//
// var functions
//

ZVAR CExtAPI::var_Create(long VarID, LPCSTR lpVarName) {
	if(!m_lpfn_var_Create) {
		ReportError("CExtAPI::var_Create error - function not found");
		return NULL;
	}

	return m_lpfn_var_Create(m_hID, VarID, lpVarName);
}
ZVAR CExtAPI::var_CreateTemp(long VarID) {
	if(!m_lpfn_var_CreateTemp) {
		ReportError("CExtAPI::var_CreateTemp error - function not found");
		return NULL;
	}

	return m_lpfn_var_CreateTemp(m_hID, VarID);
}
bool CExtAPI::var_Delete(ZVAR hVar) {
	if(!m_lpfn_var_Delete) {
		ReportError("CExtAPI::var_Delete error - function not found");
		return false;
	}

	return m_lpfn_var_Delete(m_hID, hVar);
}
bool CExtAPI::var_Rename(ZVAR hVar, LPCSTR lpNewName) {
	if(!m_lpfn_var_Rename) {
		ReportError("CExtAPI::var_Rename error - function not found");
		return false;
	}

	return m_lpfn_var_Rename(m_hID, hVar, lpNewName);
}
ZVAR CExtAPI::var_GetVar(LPCSTR lpVarName) {
	if(!m_lpfn_var_GetVar) {
		ReportError("CExtAPI::var_GetVar error - function not found");
		return NULL;
	}

	return m_lpfn_var_GetVar(m_hID, lpVarName);
}
LPCSTR CExtAPI::var_GetName(ZVAR hVar) {
	if(!m_lpfn_var_GetName) {
		ReportError("CExtAPI::var_GetName error - function not found");
		return NULL;
	}

	return m_lpfn_var_GetName(m_hID, hVar);
}
long CExtAPI::var_GetTypeID(ZVAR hVar) {
	if(!m_lpfn_var_GetTypeID) {
		ReportError("CExtAPI::var_GetTypeID error - function not found");
		return 0;
	}

	return m_lpfn_var_GetTypeID(m_hID, hVar);
}
bool CExtAPI::var_IsType(ZVAR hVar, long VarID) {
	if(!m_lpfn_var_IsType) {
		ReportError("CExtAPI::var_IsType error - function not found");
		return false;
	}

	return m_lpfn_var_IsType(m_hID, hVar, VarID);
}
bool CExtAPI::var_GetValue(ZVAR hVar, void* pValue) {
	if(!m_lpfn_var_GetValue) {
		ReportError("CExtAPI::var_GetValue error - function not found");
		return false;
	}

	return m_lpfn_var_GetValue(m_hID, hVar, pValue);
}
bool CExtAPI::var_SetValue(ZVAR hVar, void* pValue) {
	if(!m_lpfn_var_SetValue) {
		ReportError("CExtAPI::var_SetValue error - function not found");
		return false;
	}

	return m_lpfn_var_SetValue(m_hID, hVar, pValue);
}
bool CExtAPI::var_GetValueEx(ZVAR hVar, long VarID, void* pValue) {
	if(!m_lpfn_var_GetValueEx) {
		ReportError("CExtAPI::var_GetValueEx error - function not found");
		return false;
	}

	return m_lpfn_var_GetValueEx(m_hID, hVar, VarID, pValue);
}
bool CExtAPI::var_SetValueEx(ZVAR hVar, long VarID, void* pValue) {
	if(!m_lpfn_var_SetValueEx) {
		ReportError("CExtAPI::var_SetValueEx error - function not found");
		return false;
	}

	return m_lpfn_var_SetValueEx(m_hID, hVar, VarID, pValue);
}
bool CExtAPI::var_CopyValue(ZVAR hVarDest, ZVAR hVarSrc) {
	if(!m_lpfn_var_CopyValue) {
		ReportError("CExtAPI::var_CopyValue error - function not found");
		return false;
	}

	return m_lpfn_var_CopyValue(m_hID, hVarDest, hVarSrc);
}

//
// type functions
//

long CExtAPI::type_GetTypeID(LPCSTR lpTypeName) {
	if(!m_lpfn_type_GetTypeID) {
		ReportError("CExtAPI::type_GetTypeID error - function not found");
		return 0;
	}

	return m_lpfn_type_GetTypeID(m_hID, lpTypeName);
}
bool CExtAPI::type_GetTypeName(long VarID, ZVAR hStringVar) {
	if(!m_lpfn_type_GetTypeName) {
		ReportError("CExtAPI::type_GetTypeName error - function not found");
		return false;
	}

	return m_lpfn_type_GetTypeName(m_hID, VarID, hStringVar);
}

//
// string functions
//

LPCSTR CExtAPI::str_GetText(ZVAR hStr) {
	if(!m_lpfn_str_GetText) {
		ReportError("CExtAPI::str_GetText error - function not found");
		return NULL;
	}

	return m_lpfn_str_GetText(m_hID, hStr);
}
bool CExtAPI::str_SetText(ZVAR hStr, LPCSTR lpText) {
	if(!m_lpfn_str_SetText) {
		ReportError("CExtAPI::str_SetText error - function not found");
		return false;
	}

	return m_lpfn_str_SetText(m_hID, hStr, lpText);
}

//
// list functions
//

ZLIST CExtAPI::list_GetList(LPCSTR lpListName) {
	if(!m_lpfn_list_GetList) {
		ReportError("CExtAPI::list_GetList error - function not found");
		return NULL;
	}

	return m_lpfn_list_GetList(m_hID, lpListName);
}
long CExtAPI::list_nItems(ZLIST hList) {
	if(!m_lpfn_list_nItems) {
		ReportError("CExtAPI::list_nItems error - function not found");
		return 0;
	}

	return m_lpfn_list_nItems(m_hID, hList);
}
ZLIST CExtAPI::list_CreateItem(ZLIST hList, long VarID, LPCSTR lpVarName) {
	if(!m_lpfn_list_CreateItem) {
		ReportError("CExtAPI::list_CreateItem error - function not found");
		return NULL;
	}

	return m_lpfn_list_CreateItem(m_hID, hList, VarID, lpVarName);
}
ZLIST CExtAPI::list_GetItemI(ZLIST hList, long index) {
	if(!m_lpfn_list_GetItemI) {
		ReportError("CExtAPI::list_GetItemI error - function not found");
		return NULL;
	}

	return m_lpfn_list_GetItemI(m_hID, hList, index);
}
ZLIST CExtAPI::list_GetItemA(ZLIST hList, LPCSTR lpVarName) {
	if(!m_lpfn_list_GetItemA) {
		ReportError("CExtAPI::list_GetItemA error - function not found");
		return NULL;
	}

	return m_lpfn_list_GetItemA(m_hID, hList, lpVarName);
}
bool CExtAPI::list_DelItemI(ZLIST hList, long index) {
	if(!m_lpfn_list_DelItemI) {
		ReportError("CExtAPI::list_DelItemI error - function not found");
		return false;
	}

	return m_lpfn_list_DelItemI(m_hID, hList, index);
}
bool CExtAPI::list_DelItemA(ZLIST hList, LPCSTR lpVarName) {
	if(!m_lpfn_list_DelItemA) {
		ReportError("CExtAPI::list_DelItemA error - function not found");
		return false;
	}

	return m_lpfn_list_DelItemA(m_hID, hList, lpVarName);
}
bool CExtAPI::list_DelAll(ZLIST hList) {
	if(!m_lpfn_list_GetList) {
		ReportError("CExtAPI::list_DelAll error - function not found");
		return false;
	}

	return m_lpfn_list_DelAll(m_hID, hList);
}

//
// map functions
//

ZMAP CExtAPI::map_GetMap(LPCSTR lpMapName) {
	if(!m_lpfn_map_GetMap) {
		ReportError("CExtAPI::map_GetMap error - function not found");
		return NULL;
	}

	return m_lpfn_map_GetMap(m_hID, lpMapName);
}

bool CExtAPI::map_Init(ZMAP hMap, long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag) {
	if(!m_lpfn_map_Init) {
		ReportError("CExtAPI::map_Init error - function not found");
		return false;
	}

	return m_lpfn_map_Init(m_hID, hMap, nx, ny, MapTypeID, HorizScale, WrapFlag);
}
bool CExtAPI::map_Free(ZMAP hMap) {
	if(!m_lpfn_map_Free) {
		ReportError("CExtAPI::map_Free error - function not found");
		return false;
	}

	return m_lpfn_map_Free(m_hID, hMap);
}
long CExtAPI::map_GetMapType(ZMAP hMap) {
	if(!m_lpfn_map_GetMapType) {
		ReportError("CExtAPI::map_GetMapType error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMapType(m_hID, hMap);
}
long CExtAPI::map_GetPixelSize(ZMAP hMap) {
	if(!m_lpfn_map_GetPixelSize) {
		ReportError("CExtAPI::map_GetPixelSize error - function not found");
		return 0;
	}

	return m_lpfn_map_GetPixelSize(m_hID, hMap);
}
bool CExtAPI::map_GetWrapFlag(ZMAP hMap) {
	if(!m_lpfn_map_GetWrapFlag) {
		ReportError("CExtAPI::map_GetWrapFlag error - function not found");
		return false;
	}

	return m_lpfn_map_GetWrapFlag(m_hID, hMap);
}
long CExtAPI::map_GetWidth(ZMAP hMap) {
	if(!m_lpfn_map_GetWidth) {
		ReportError("CExtAPI::map_GetWidth error - function not found");
		return 0;
	}

	return m_lpfn_map_GetWidth(m_hID, hMap);
}
long CExtAPI::map_GetHeight(ZMAP hMap) {
	if(!m_lpfn_map_GetHeight) {
		ReportError("CExtAPI::map_GetHeight error - function not found");
		return 0;
	}

	return m_lpfn_map_GetHeight(m_hID, hMap);
}
bool CExtAPI::map_GetPixel(ZMAP hMap, long x, long y, void* pValue) {
	if(!m_lpfn_map_GetPixel) {
		ReportError("CExtAPI::map_GetPixel error - function not found");
		return false;
	}

	return m_lpfn_map_GetPixel(m_hID, hMap, x, y, pValue);
}
bool CExtAPI::map_SetPixel(ZMAP hMap, long x, long y, void* pValue) {
	if(!m_lpfn_map_SetPixel) {
		ReportError("CExtAPI::map_SetPixel error - function not found");
		return false;
	}

	return m_lpfn_map_SetPixel(m_hID, hMap, x, y, pValue);
}
bool CExtAPI::map_GetMinMaxAlt(ZMAP hMap, float& minval, float& maxval) {
	if(!m_lpfn_map_GetMinMaxAlt) {
		ReportError("CExtAPI::map_GetMinMaxAlt error - function not found");
		return false;
	}

	return m_lpfn_map_GetMinMaxAlt(m_hID, hMap, minval, maxval);
}
float CExtAPI::map_GetHorizScale(ZMAP hMap) {
	if(!m_lpfn_map_GetHorizScale) {
		ReportError("CExtAPI::map_GetHorizScale error - function not found");
		return false;
	}

	return m_lpfn_map_GetHorizScale(m_hID, hMap);
}
bool CExtAPI::map_GetMosaicFlag(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicFlag) {
		ReportError("CExtAPI::map_GetMosaicFlag error - function not found");
		return false;
	}

	return m_lpfn_map_GetMosaicFlag(m_hID, hMap);
}
long CExtAPI::map_GetMosaicTileSize(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicTileSize) {
		ReportError("CExtAPI::map_GetMosaicTileSize error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMosaicTileSize(m_hID, hMap);
}
long CExtAPI::map_GetMosaicTilesX(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicTilesX) {
		ReportError("CExtAPI::map_GetMosaicTilesX error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMosaicTilesX(m_hID, hMap);
}
long CExtAPI::map_GetMosaicTilesY(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicTilesY) {
		ReportError("CExtAPI::map_GetMosaicTilesY error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMosaicTilesY(m_hID, hMap);
}

//
// mosaic tile functions
//

bool CExtAPI::tile_GetPixel(LPVOID hTile, long x, long y, void* pValue) {
	if(!m_lpfn_tile_GetPixel) {
		ReportError("CExtAPI::tile_GetPixel error - function not found");
		return false;
	}

	return m_lpfn_tile_GetPixel(m_hID, hTile, x, y, pValue);
}
bool CExtAPI::tile_SetPixel(LPVOID hTile, long x, long y, void* pValue) {
	if(!m_lpfn_tile_SetPixel) {
		ReportError("CExtAPI::tile_SetPixel error - function not found");
		return false;
	}

	return m_lpfn_tile_SetPixel(m_hID, hTile, x, y, pValue);
}
bool CExtAPI::tile_GetMinMaxAlt(LPVOID hTile, float& minval, float& maxval) {
	if(!m_lpfn_tile_GetMinMaxAlt) {
		ReportError("CExtAPI::tile_GetMinMaxAlt error - function not found");
		return false;
	}

	return m_lpfn_tile_GetMinMaxAlt(m_hID, hTile, minval, maxval);
}

//
// format functions
//

ZFORMAT CExtAPI::format_Create(LPCTSTR lpMapName, LPCSTR lpLongName, LPCSTR lpFileExt, LPCSTR lpNativeProgramName) {
	if(!m_lpfn_format_Create) {
		ReportError("CExtAPI::format_Create error - function not found");
		return NULL;
	}

	return m_lpfn_format_Create(m_hID, lpMapName, lpLongName, lpFileExt, lpNativeProgramName);
}
ZFORMAT CExtAPI::format_CreateGeneric(long MapTypeID, LPCTSTR lpLongName, LPCSTR lpFileExt, LPCSTR lpNativeProgramName) {
	if(!m_lpfn_format_Create) {
		ReportError("CExtAPI::format_CreateGeneric error - function not found");
		return NULL;
	}

	return m_lpfn_format_CreateGeneric(m_hID, MapTypeID, lpLongName, lpFileExt, lpNativeProgramName);
}
bool CExtAPI::format_SetFlags(ZFORMAT hFormat, bool Native, bool Mosaic) {
	if(!m_lpfn_format_SetFlags) {
		ReportError("CExtAPI::format_SetFlags error - function not found");
		return false;
	}

	return m_lpfn_format_SetFlags(m_hID, hFormat, Native, Mosaic);
}
LPCSTR CExtAPI::format_GetExt(ZFORMAT hFormat) {
	if(!m_lpfn_format_GetExt) {
		ReportError("CExtAPI::format_GetExt error - function not found");
		return NULL;
	}

	return m_lpfn_format_GetExt(m_hID, hFormat);
}
ZLIST CExtAPI::format_GetOptionList(ZFORMAT hFormat) {
	if(!m_lpfn_format_GetOptionList) {
		ReportError("CExtAPI::format_GetOptionList error - function not found");
		return NULL;
	}

	return m_lpfn_format_GetOptionList(m_hID, hFormat);
}
bool CExtAPI::format_SetOptionValue(ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue) {
	if(!m_lpfn_format_SetOptionValue) {
		ReportError("CExtAPI::format_SetOptionValue error - function not found");
		return false;
	}

	return m_lpfn_format_SetOptionValue(m_hID, hFormat, lpOptionName, VarID, pValue);
}
bool CExtAPI::format_GetOptionValue(ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue) {
	if(!m_lpfn_format_GetOptionValue) {
		ReportError("CExtAPI::format_GetOptionValue error - function not found");
		return false;
	}

	return m_lpfn_format_GetOptionValue(m_hID, hFormat, lpOptionName, VarID, pValue);
}

//
// prog box functions
//

bool CExtAPI::progbox_ShowWnd(ZVAR hProgWnd) {
	if(!m_lpfn_progbox_ShowWnd) {
		ReportError("CExtAPI::progbox_ShowWnd error - function not found");
		return false;
	}

	return m_lpfn_progbox_ShowWnd(m_hID, hProgWnd);
}
bool CExtAPI::progbox_HideWnd(ZVAR hProgWnd) {
	if(!m_lpfn_progbox_HideWnd) {
		ReportError("CExtAPI::progbox_HideWnd error - function not found");
		return false;
	}

	return m_lpfn_progbox_HideWnd(m_hID, hProgWnd);
}
bool CExtAPI::progbox_SetTitle(ZVAR hProgWnd, LPCSTR lpTitleText) {
	if(!m_lpfn_progbox_SetTitle) {
		ReportError("CExtAPI::progbox_SetTitle error - function not found");
		return false;
	}

	return m_lpfn_progbox_SetTitle(m_hID, hProgWnd, lpTitleText);
}
bool CExtAPI::progbox_SetProgress(ZVAR hProgWnd, __int64 p, __int64 pmax) {
	if(!m_lpfn_progbox_SetProgress) {
		ReportError("CExtAPI::progbox_SetProgress error - function not found");
		return false;
	}

	return m_lpfn_progbox_SetProgress(m_hID, hProgWnd, p, pmax);
}

//
// combo box functions
//

bool CExtAPI::combosel_SetOptions(ZVAR hComboSel, LPCSTR lpOptionsStr) {
	if(!m_lpfn_combosel_SetOptions) {
		ReportError("CExtAPI::combosel_SetOptions error - function not found");
		return false;
	}

	return m_lpfn_combosel_SetOptions(m_hID, hComboSel, lpOptionsStr);
}
LPCSTR CExtAPI::combosel_GetOptions(ZVAR hComboSel) {
	if(!m_lpfn_combosel_GetOptions) {
		ReportError("CExtAPI::combosel_GetOptions error - function not found");
		return false;
	}

	return m_lpfn_combosel_GetOptions(m_hID, hComboSel);
}
bool CExtAPI::combosel_SetCurSel(ZVAR hComboSel, LPCSTR lpCurSelStr) {
	if(!m_lpfn_combosel_SetCurSel) {
		ReportError("CExtAPI::combosel_SetCurSel error - function not found");
		return false;
	}

	return m_lpfn_combosel_SetCurSel(m_hID, hComboSel, lpCurSelStr);
}
LPCSTR CExtAPI::combosel_GetCurSel(ZVAR hComboSel) {
	if(!m_lpfn_combosel_GetCurSel) {
		ReportError("CExtAPI::combosel_GetCurSel error - function not found");
		return false;
	}

	return m_lpfn_combosel_GetCurSel(m_hID, hComboSel);
}

//
// buffer functions
//

bool CExtAPI::buffer_Init(ZVAR hBuf, long nItems, long ItemSize) {
	if(!m_lpfn_buffer_Init) {
		ReportError("CExtAPI::buffer_Init error - function not found");
		return false;
	}

	return m_lpfn_buffer_Init(m_hID, hBuf, nItems, ItemSize);
}
bool CExtAPI::buffer_Free(ZVAR hBuf) {
	if(!m_lpfn_buffer_Free) {
		ReportError("CExtAPI::buffer_Free error - function not found");
		return false;
	}

	return m_lpfn_buffer_Free(m_hID, hBuf);
}
long CExtAPI::buffer_nItems(ZVAR hBuf) {
	if(!m_lpfn_buffer_nItems) {
		ReportError("CExtAPI::buffer_nItems error - function not found");
		return 0;
	}

	return m_lpfn_buffer_nItems(m_hID, hBuf);
}
long CExtAPI::buffer_MemSize(ZVAR hBuf) {
	if(!m_lpfn_buffer_MemSize) {
		ReportError("CExtAPI::buffer_MemSize error - function not found");
		return 0;
	}

	return m_lpfn_buffer_MemSize(m_hID, hBuf);
}
long CExtAPI::buffer_ItemSize(ZVAR hBuf) {
	if(!m_lpfn_buffer_ItemSize) {
		ReportError("CExtAPI::buffer_ItemSize error - function not found");
		return 0;
	}

	return m_lpfn_buffer_ItemSize(m_hID, hBuf);
}
LPVOID CExtAPI::buffer_GetPtr(ZVAR hBuf) {
	if(!m_lpfn_buffer_GetPtr) {
		ReportError("CExtAPI::buffer_GetPtr error - function not found");
		return NULL;
	}

	return m_lpfn_buffer_GetPtr(m_hID, hBuf);
}
bool CExtAPI::buffer_GetValue(ZVAR hBuf, long index, void* pValue) {
	if(!m_lpfn_buffer_GetValue) {
		ReportError("CExtAPI::buffer_GetValue error - function not found");
		return false;
	}

	return m_lpfn_buffer_GetValue(m_hID, hBuf, index, pValue);
}
bool CExtAPI::buffer_SetValue(ZVAR hBuf, long index, void* pValue) {
	if(!m_lpfn_buffer_SetValue) {
		ReportError("CExtAPI::buffer_SetValue error - function not found");
		return false;
	}

	return m_lpfn_buffer_SetValue(m_hID, hBuf, index, pValue);
}

//
// project functions
//

bool CExtAPI::project_GetHeightfieldRange(float& minval, float& maxval) {
	if(!m_lpfn_project_GetHeightfieldRange) {
		ReportError("CExtAPI::project_GetHeightfieldRange error - function not found");
		return false;
	}

	return m_lpfn_project_GetHeightfieldRange(m_hID, minval, maxval);
}

// calculation functions


COLORREF CExtAPI::calc_AM_GetPixelColour(AttributesPixel AMpxl) {
	if(!m_lpfn_calc_AM_GetPixelColour) {
		ReportError("CExtAPI::calc_AM_GetPixelColour error - function not found");
		return RGB(0, 0, 0);
	}

	return m_lpfn_calc_AM_GetPixelColour(m_hID, AMpxl);
}

AttributesPixel CExtAPI::calc_AM_GetIDfromColour(COLORREF col) {

	if(!m_lpfn_calc_AM_GetIDfromColour) {
		ReportError("CExtAPI::calc_AM_GetIDfromColour error - function not found");
		
		AttributesPixel AMpxl;
		AMpxl.LandTypeID = 0;
		AMpxl.ClimateID = 0;
		return AMpxl;
	}

	return m_lpfn_calc_AM_GetIDfromColour(m_hID, col);
}

//
// file/dir functions
//

LPCSTR	CExtAPI::file_GetExt(LPCSTR lpFileName) {
	if(!m_lpfn_file_GetExt) {
		ReportError("CExtAPI::file_GetExt error - function not found");
		return false;
	}

	return m_lpfn_file_GetExt(m_hID, lpFileName);
}
bool CExtAPI::file_FileExists(LPCSTR lpFileName) {
	if(!m_lpfn_file_FileExists) {
		ReportError("CExtAPI::file_FileExists error - function not found");
		return false;
	}

	return m_lpfn_file_FileExists(m_hID, lpFileName);
}

//
// extension functions
//

ZVAR CExtAPI::zeofunc_LoadFunc(LPCSTR lpFnName, long rvalVarID, ZLIST hArgList) {
	if(!m_lpfn_zeofunc_LoadFunc) {
		ReportError("CExtAPI::zeofunc_LoadFunc error - function not found");
		return NULL;
	}

	return m_lpfn_zeofunc_LoadFunc(m_hID, lpFnName, rvalVarID, hArgList);
}
ZVAR CExtAPI::zeofunc_LoadFuncEx(LPCSTR lpFnName, LPCSTR lpExportName, long rvalVarID, ZLIST hArgList) {
	if(!m_lpfn_zeofunc_LoadFuncEx) {
		ReportError("CExtAPI::zeofunc_LoadFuncEx error - function not found");
		return NULL;
	}

	return m_lpfn_zeofunc_LoadFuncEx(m_hID, lpFnName, lpExportName, rvalVarID, hArgList);
}
ZVAR CExtAPI::zeofunc_GetFunc(LPCSTR lpFnName) {
	if(!m_lpfn_zeofunc_GetFunc) {
		ReportError("CExtAPI::zeofunc_GetFunc error - function not found");
		return NULL;
	}

	return m_lpfn_zeofunc_GetFunc(m_hID, lpFnName);
}
long CExtAPI::zeofunc_GetReturnTypeID(ZFUNC hFunc) {
	if(!m_lpfn_zeofunc_LoadFunc) {
		ReportError("CExtAPI::zeofunc_GetReturnTypeID error - function not found");
		return 0;
	}

	return m_lpfn_zeofunc_GetReturnTypeID(m_hID, hFunc);
}
bool CExtAPI::zeofunc_GetArgListPrototype(ZFUNC hFunc, ZLIST hArgList) {
	if(!m_lpfn_zeofunc_GetArgListPrototype) {
		ReportError("CExtAPI::zeofunc_GetArgListPrototype error - function not found");
		return false;
	}

	return m_lpfn_zeofunc_GetArgListPrototype(m_hID, hFunc, hArgList);
}
ZVAR CExtAPI::zeofunc_Execute(ZFUNC hFunc, ZLIST hArgList) {
	if(!m_lpfn_zeofunc_Execute) {
		ReportError("CExtAPI::zeofunc_Execute error - function not found");
		return NULL;
	}

	return m_lpfn_zeofunc_Execute(m_hID, hFunc, hArgList);
}

//
// bootstrapping function
//

FARPROC CExtAPI::GetFuncHandle(LPCSTR lpFuncName) {
	if(!m_lpfnGetFuncHandle) {
		ReportError("CExtAPI::GetFuncHandle error - API not initialised");
		return NULL;
	}

	FARPROC hProc = m_lpfnGetFuncHandle(m_hID, lpFuncName);
	if(!hProc) {
		ReportError("CExtAPI::GetFuncHandle error - Function not found");
		return NULL;
	}

	return hProc;
}